package graphical;

import java.awt.EventQueue;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.border.EmptyBorder;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import com.jgoodies.forms.layout.FormLayout;
import com.jgoodies.forms.layout.ColumnSpec;
import com.jgoodies.forms.layout.RowSpec;
import com.jgoodies.forms.factories.FormFactory;
import javax.swing.JLabel;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;

import javax.swing.JOptionPane;
import javax.swing.JTextField;
import javax.swing.JButton;
import javax.swing.JPasswordField;

@SuppressWarnings("serial")
public class LoginFrame extends JFrame {

	private JPanel contentPane;
	private JTextField tfSignInUsername;
	private JPasswordField pfSignInPassword;

	private JTextField tfSignUpFullName;
	private JTextField tfSignUpEmail;
	private JTextField tfSignUpUsername;

	private JPasswordField pfSignUpPassword;

	// Maintain a list of listeners to this form which will be the client to
	// know when the buttons are pressed
	ArrayList<LogInEventListener> listeners;

	/**
	 * Launch the application.
	 */
	public static void main(String[] args) {
		EventQueue.invokeLater(new Runnable() {
			public void run() {
				try {
					LoginFrame frame = new LoginFrame();
					frame.setVisible(true);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		});
	}

	public void addEventListener(LogInEventListener listener) {
		listeners.add(listener);
	}

	public void removeEventListener(LogInEventListener listener) {
		listeners.remove(listener);
	}

	/**
	 * Create the frame.
	 */
	public LoginFrame() {
		listeners = new ArrayList<LogInEventListener>();

		setTitle("Welcome to Twitter!!");
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		setBounds(100, 100, 548, 388);
		contentPane = new JPanel();
		contentPane.setBorder(new EmptyBorder(5, 5, 5, 5));
		setContentPane(contentPane);

		GridBagLayout gbl_contentPane = new GridBagLayout();
		gbl_contentPane.columnWidths = new int[] { 0, 0 };
		gbl_contentPane.rowHeights = new int[] { 0, 0, 0 };
		gbl_contentPane.columnWeights = new double[] { 0.0, Double.MIN_VALUE };
		gbl_contentPane.rowWeights = new double[] { 0.0, 1.0, Double.MIN_VALUE };
		contentPane.setLayout(gbl_contentPane);

		JPanel pnlSignin = new JPanel();
		GridBagConstraints gbc_panel = new GridBagConstraints();
		gbc_panel.anchor = GridBagConstraints.WEST;
		gbc_panel.insets = new Insets(0, 0, 5, 0);
		gbc_panel.fill = GridBagConstraints.VERTICAL;
		gbc_panel.gridx = 0;
		gbc_panel.gridy = 0;
		contentPane.add(pnlSignin, gbc_panel);
		pnlSignin
				.setLayout(new FormLayout(new ColumnSpec[] {
						FormFactory.RELATED_GAP_COLSPEC,
						FormFactory.DEFAULT_COLSPEC,
						FormFactory.RELATED_GAP_COLSPEC,
						ColumnSpec.decode("default:grow"), }, new RowSpec[] {
						FormFactory.RELATED_GAP_ROWSPEC,
						FormFactory.DEFAULT_ROWSPEC,
						FormFactory.RELATED_GAP_ROWSPEC,
						FormFactory.DEFAULT_ROWSPEC,
						FormFactory.RELATED_GAP_ROWSPEC,
						FormFactory.DEFAULT_ROWSPEC, }));

		JLabel lblUsername = new JLabel("Username");
		lblUsername.setFont(new Font("Calibri", Font.PLAIN, 16));
		pnlSignin.add(lblUsername, "2, 2, right, default");

		tfSignInUsername = new JTextField("w.rooney");
		tfSignInUsername.setFont(new Font("Calibri", Font.PLAIN, 16));
		pnlSignin.add(tfSignInUsername, "4, 2, fill, default");
		tfSignInUsername.setColumns(10);

		JLabel lblPassword = new JLabel("pass");
		lblPassword.setFont(new Font("Calibri", Font.PLAIN, 16));
		pnlSignin.add(lblPassword, "2, 4, right, default");

		pfSignInPassword = new JPasswordField("pass");
		pfSignInPassword.setFont(new Font("Calibri", Font.PLAIN, 16));
		pnlSignin.add(pfSignInPassword, "4, 4, fill, default");

		JButton btnSignIn = new JButton("Sign In");
		btnSignIn.addActionListener(new ActionListener() {

			@SuppressWarnings("deprecation")
			@Override
			public void actionPerformed(ActionEvent arg0) {

				// Do checking if the Sign In fields are filled, then go to
				// through the list of listeners and inform them that the sign
				// in button is pressed
				if (tfSignInUsername.getText().trim().isEmpty()) {
					JOptionPane.showMessageDialog(null,
							"Please enter your Username before signing in.");
					return;
				}
				if (pfSignInPassword.getText().trim().isEmpty()) {
					JOptionPane.showMessageDialog(null,
							"Please enter your Password before signing in.");
					return;
				}

				for (LogInEventListener listener : listeners) {

					if (!listener.btnSignInPressed(tfSignInUsername.getText()
							.trim(), pfSignInPassword.getText().trim()))
						JOptionPane.showMessageDialog(null,
								"The username and password do not match!");
				}
			}
		});
		btnSignIn.setFont(new Font("Calibri", Font.BOLD, 14));
		pnlSignin.add(btnSignIn, "4, 6");

		JPanel pnlSignUp = new JPanel();
		GridBagConstraints gbc_panel_1 = new GridBagConstraints();
		gbc_panel_1.fill = GridBagConstraints.BOTH;
		gbc_panel_1.gridx = 0;
		gbc_panel_1.gridy = 1;
		contentPane.add(pnlSignUp, gbc_panel_1);
		pnlSignUp
				.setLayout(new FormLayout(new ColumnSpec[] {
						FormFactory.RELATED_GAP_COLSPEC,
						FormFactory.DEFAULT_COLSPEC,
						FormFactory.RELATED_GAP_COLSPEC,
						ColumnSpec.decode("default:grow"), }, new RowSpec[] {
						FormFactory.RELATED_GAP_ROWSPEC,
						FormFactory.DEFAULT_ROWSPEC,
						FormFactory.RELATED_GAP_ROWSPEC,
						FormFactory.DEFAULT_ROWSPEC,
						FormFactory.RELATED_GAP_ROWSPEC,
						FormFactory.DEFAULT_ROWSPEC,
						FormFactory.RELATED_GAP_ROWSPEC,
						FormFactory.DEFAULT_ROWSPEC,
						FormFactory.RELATED_GAP_ROWSPEC,
						FormFactory.DEFAULT_ROWSPEC,
						FormFactory.RELATED_GAP_ROWSPEC,
						FormFactory.DEFAULT_ROWSPEC,
						FormFactory.RELATED_GAP_ROWSPEC,
						FormFactory.DEFAULT_ROWSPEC,
						FormFactory.RELATED_GAP_ROWSPEC,
						FormFactory.DEFAULT_ROWSPEC, }));

		JLabel lblNewToTwitter = new JLabel("New to twitter? Sign Up now!!");
		lblNewToTwitter.setFont(new Font("Calibri", Font.BOLD, 16));
		pnlSignUp.add(lblNewToTwitter, "4, 4, default, center");

		JLabel lblFullName = new JLabel("Full Name");
		lblFullName.setFont(new Font("Calibri", Font.PLAIN, 16));
		pnlSignUp.add(lblFullName, "2, 8, right, default");

		tfSignUpFullName = new JTextField();
		tfSignUpFullName.setFont(new Font("Calibri", Font.PLAIN, 16));
		tfSignUpFullName.setColumns(10);
		pnlSignUp.add(tfSignUpFullName, "4, 8, fill, default");

		JLabel lblEmail = new JLabel("Email");
		lblEmail.setFont(new Font("Calibri", Font.PLAIN, 16));
		pnlSignUp.add(lblEmail, "2, 10, right, center");

		tfSignUpEmail = new JTextField();
		tfSignUpEmail.setFont(new Font("Calibri", Font.PLAIN, 16));
		tfSignUpEmail.setColumns(10);
		pnlSignUp.add(tfSignUpEmail, "4, 10, fill, default");

		JLabel lblUsername_1 = new JLabel("Username");
		lblUsername_1.setFont(new Font("Calibri", Font.PLAIN, 16));
		pnlSignUp.add(lblUsername_1, "2, 12, right, default");

		tfSignUpUsername = new JTextField();
		tfSignUpUsername.setFont(new Font("Calibri", Font.PLAIN, 16));
		tfSignUpUsername.setColumns(10);
		pnlSignUp.add(tfSignUpUsername, "4, 12, fill, default");

		JLabel lblPassword_1 = new JLabel("Password");
		lblPassword_1.setFont(new Font("Calibri", Font.PLAIN, 16));
		pnlSignUp.add(lblPassword_1, "2, 14, right, default");

		pfSignUpPassword = new JPasswordField();
		pfSignUpPassword.setFont(new Font("Calibri", Font.PLAIN, 16));
		pnlSignUp.add(pfSignUpPassword, "4, 14, fill, default");

		JButton btnSignUp = new JButton("Sign Up");
		btnSignUp.setFont(new Font("Calibri", Font.BOLD, 14));
		pnlSignUp.add(btnSignUp, "4, 16");
		btnSignUp.addActionListener(new ActionListener() {

			@SuppressWarnings("deprecation")
			@Override
			public void actionPerformed(ActionEvent arg0) {

				// Do checking if the Sign Up fields are filled, then go to
				// through the list of listeners and inform them that the sign
				// up button is pressed
				if (tfSignUpFullName.getText().trim().isEmpty()) {
					JOptionPane.showMessageDialog(null,
							"Please enter your Full Name before signing up.");
					return;
				}
				if (tfSignUpFullName.getText().trim().split(" ").length < 2) {
					JOptionPane
							.showMessageDialog(
									null,
									"Please enter your Last Name along with your First Name in the Full Name text box.");
					return;
				}
				if (tfSignUpUsername.getText().trim().isEmpty()) {
					JOptionPane.showMessageDialog(null,
							"Please enter your Username before signing up.");
					return;
				}
				if (pfSignUpPassword.getText().isEmpty()) {
					JOptionPane.showMessageDialog(null,
							"Please enter your Password before signing up.");
					return;
				}
				for (LogInEventListener listener : listeners) {
					if (!listener.btnSignUpPressed(tfSignUpUsername.getText()
							.trim(),
							tfSignUpFullName.getText().trim().split(" ")[0],
							tfSignUpFullName.getText().trim().split(" ")[1],
							tfSignUpEmail.getText().trim(), pfSignUpPassword
									.getText()))
						JOptionPane
								.showMessageDialog(null,
										"The Username is already taken, try another one.");

				}
			}
		});
	}

}
