package graphical;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;
import javax.swing.ListSelectionModel;
import javax.swing.ScrollPaneConstants;
import javax.swing.SwingConstants;

import java.awt.GridBagLayout;
import javax.swing.JLabel;
import javax.swing.border.Border;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableColumn;
import javax.xml.crypto.Data;

import rmi.Tweets;
import rmi.Users;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.Font;
import java.awt.LayoutManager;
import java.awt.LayoutManager2;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.Iterator;
import java.util.Vector;

import javax.swing.JScrollBar;
import java.awt.Scrollbar;
import javax.swing.JTable;
import java.awt.FlowLayout;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

@SuppressWarnings("serial")
public class MainPanel extends JPanel {
	private JTable table;
	private static final int CHECK_COL = 3;

	ArrayList<MainPanelEventListener> listeners;

	public void addEventListener(MainPanelEventListener listener) {
		listeners.add(listener);
	}

	@SuppressWarnings("unchecked")
	public <T> MainPanel(ArrayList<?> data, Class<T> clazz) {
		setLayout(new BorderLayout(0, 0));

		listeners = new ArrayList<MainPanelEventListener>();

		table = new JTable();
		add(table, BorderLayout.CENTER);
		table.setColumnSelectionAllowed(false);
		table.setDragEnabled(false);
		table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		table.getTableHeader().setFont(new Font("SansSerif", Font.BOLD, 15));

		JScrollPane pane = new JScrollPane(table);
		pane.setFont(new Font("SansSerif", Font.PLAIN, 13));
		add(pane, BorderLayout.CENTER);

		DefaultTableModel model = new DefaultTableModel() {
			@Override
			public boolean isCellEditable(int row, int column) {
				return false;
			}
		};
		table.setModel(model);

		if (data == null || data.size() == 0) {

			model.addColumn("null");
			model.addColumn("NO DATA FOUND");

			if (clazz.equals(Users.class))
				model.addRow(new String[] { null,
						"There are no Users available" });
			else
				model.addRow(new String[] { null,
						"There are no TWEETS available" });

			table.getColumnModel().getColumn(0).setMinWidth(0);
			table.getColumnModel().getColumn(0).setMaxWidth(0);
			table.getColumnModel().getColumn(0).setWidth(0);
			return;
		}
		table.getTableHeader().setReorderingAllowed(false);

		if (clazz.equals(Users.class)) {

			Object[][] DATA = new Object[data.size()][3];
			String[] COLUMNS = { "user", "User Name", "Status", "Following?" };
			int i = 0;
			for (Users user : (ArrayList<Users>) data) {
				DATA[i] = new Object[] { user.getUsername(), user.getName(),
						user.getStatus(), user.isFollowed()};
				i++;

			}
			DataModel dataModel = new DataModel(DATA, COLUMNS);
			table.setModel(dataModel);

		} else {

			model.addColumn("user");
			model.addColumn("MessageID");
			model.addColumn("User Name");
			model.addColumn("Tweet");
			model.addColumn("Date");
			model.addColumn("Retweeted By");

			for (Tweets tweet : (ArrayList<Tweets>) data) {

				// Code to get the difference between current time and the date
				// of
				// the tweet
				try {
					SimpleDateFormat format = new SimpleDateFormat(
							"yyyy-MM-dd HH:mm:ss");

					Calendar cal = Calendar.getInstance();
					Date now = format.parse(format.format(cal.getTime()));
					//
					String dateWithoutMicroSec = tweet.getDate().substring(0,
							tweet.getDate().length() - 2);
					Date tweetDate = format.parse(dateWithoutMicroSec);
					// in milliseconds
					long diff = now.getTime() - tweetDate.getTime();

					long diffMinutes = diff / (60 * 1000) % 60;
					long diffHours = diff / (60 * 60 * 1000) % 24;
					long diffDays = diff / (24 * 60 * 60 * 1000);

					String displayDate;
					if (diffDays > 0)
						displayDate = String.valueOf(diffDays) + " days ago";
					else if (diffHours > 0)
						displayDate = String.valueOf(diffHours) + " hours ago";
					else if (diffMinutes > 0)
						displayDate = String.valueOf(diffMinutes)
								+ " minutes ago";
					else
						displayDate = "NOW";
					model.addRow(new Object[] {
							tweet.getOriginalTweeterUserName(),
							tweet.getMessageID(), tweet.getOriginalTweeter(),
							tweet.getMessage(), displayDate,
							tweet.getRetweetedBy() });
				} catch (Exception e) {
					System.out.println(e.toString());
				}
				// Hide message ID
				table.getColumnModel().getColumn(0).setMinWidth(0);
				table.getColumnModel().getColumn(0).setMaxWidth(0);
				table.getColumnModel().getColumn(0).setWidth(0);
				table.getColumnModel().getColumn(1).setMinWidth(0);
				table.getColumnModel().getColumn(1).setMaxWidth(0);
				table.getColumnModel().getColumn(1).setWidth(0);

				table.getColumnModel().getColumn(2).setMaxWidth(150);
				table.getColumnModel().getColumn(2).setMinWidth(100);
				table.getColumnModel().getColumn(5).setMaxWidth(150);
				table.getColumnModel().getColumn(5).setMinWidth(100);
				table.getColumnModel().getColumn(4).setMaxWidth(100);
			}
		}
		table.addMouseListener(new MouseAdapter() {
			public void mouseClicked(MouseEvent e) {
				if (e.getClickCount() == 2) {
					String userSelected = (String) table.getValueAt(
							table.getSelectedRow(), 0);
					if (userSelected == null)
						return;

					for (MainPanelEventListener listener : listeners)
						listener.userDoubleClicked(userSelected);
				}
			}

			@Override
			public void mouseReleased(MouseEvent e) {

				int messageID;
				String userName;    //**
				try {
					messageID = Integer.parseInt((String) table.getValueAt(
							table.getSelectedRow(), 1));
					userName = (String) table.getValueAt(table.getSelectedRow(), 0);  //get the username of the selected user
				} catch (Exception ex) {
					return;
				}
				if (messageID == 0 || userName == null)  //**
					return;

				int r = table.rowAtPoint(e.getPoint());
				if (r >= 0 && r < table.getRowCount()) {
					table.setRowSelectionInterval(r, r);
				} else {
					table.clearSelection();
				}

				int rowindex = table.getSelectedRow();
				if (rowindex < 0)
					return;
				if (e.isPopupTrigger()
						&& e.getComponent() instanceof JTable) {  //the user clicked retweet
								JPopupMenu popup = new PopUpDemo(userName,messageID); 
						popup.show(e.getComponent(), e.getX(), e.getY());
				}
			}
		});
	}

	private class DataModel extends DefaultTableModel {

		public DataModel(Object[][] data, Object[] columnNames) {
			super(data, columnNames);
		}

		@Override
		public Class<?> getColumnClass(int columnIndex) {
			if (columnIndex == CHECK_COL) {
				return getValueAt(0, CHECK_COL).getClass();
			}
			return super.getColumnClass(columnIndex);
		}

		// Get what row and column is pressed
		@Override
		public boolean isCellEditable(int row, int column) {
			if (column == CHECK_COL) {
				String userSelected = (String) getValueAt(row, 0);
				for (MainPanelEventListener listener : listeners)
					listener.btnFollowPressed(userSelected);
			}
			return column == CHECK_COL;
		}
	}

	class PopUpDemo extends JPopupMenu {
		JMenuItem itemRetweet;
		JMenuItem itemPrivateMessage;
		
		public PopUpDemo(final String userName,final Integer messageID) {
			
			itemRetweet = new JMenuItem("Re-Tweet!!");			
			itemRetweet.addMouseListener(new MouseAdapter() {
				public void mousePressed(MouseEvent e) {
					for (MainPanelEventListener listener : listeners)
						listener.btnRetweetClicked(messageID);
				}
			});
			itemPrivateMessage = new JMenuItem("Send Private Message");
			itemPrivateMessage.addMouseListener(new MouseAdapter() {
				public void mousePressed(MouseEvent e){
					
					for(MainPanelEventListener listener: listeners){
						
						listener.btnPrivateMessageClicked(userName);
					}
				}
			}); 
			add(itemRetweet);	
			add(itemPrivateMessage);
			
		}
	}
}