package rmi;

import graphical.MainFrame;

import java.lang.reflect.Array;
import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.rmi.server.UnicastRemoteObject;
import java.util.ArrayList;
import java.util.HashMap;

public class Client extends UnicastRemoteObject implements ClientInterface {
	private static final long serialVersionUID = 1L;

	public String username, name;
	// ArrayList storing the username and the name of the users
	public ArrayList<Users> followers;
	public ArrayList<Users> following;

	public static ServerInteface server;

	public Client() throws RemoteException {
		super();
		followers = new ArrayList<Users>();
		following = new ArrayList<Users>();

		String srvName = "Server";
		Registry registry = null;
		try {
			registry = LocateRegistry.getRegistry(5010);
			server = (ServerInteface) registry.lookup(srvName);

		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	@Override
	public String getUsername() throws RemoteException {
		return username;

	}

	@Override
	public void setUsername(String username) throws RemoteException {
		this.username = username;
	}

	@Override
	public String getName() throws RemoteException {
		return name;
	}

	@Override
	public void setName(String name) throws RemoteException {
		this.name = name;

	}

	@Override
	public void getTweet(Users u, String message, String date)
			throws RemoteException {
		// System.out.println("Tweet from: " + u.getUsername() + "(" +
		// u.getName() + ") date: "
		// + date + ": " + message + "\n\n");
		MainFrame.recieveNewTweet();

	}

	@Override
	public void getReTweet(Users u, String originalUser, String originalName,
			String message, String date) throws RemoteException {

		// System.out.println("Retweet from: " + originalUser + " by: " +
		// u.getUsername()
		// + " date: " + date + ": " + message + "\n\n");
		MainFrame.recieveNewTweet();
	}

	@Override
	public void getPrivateMessage(Users u, String message, String date)
			throws RemoteException {
		// System.out.println("Private Message from: " + u.getUsername() + "\t'"
		// + message + "' sent:" + date);
		MainFrame.recieveNewPrivateMessage();
	}

	@Override
	public void followUser(Users u) throws RemoteException {
		following.add(u);
	}

	@Override
	public ArrayList<Users> getFollowers() throws RemoteException {
		return followers;
	}

	@Override
	public ArrayList<Users> getFollowing() throws RemoteException {
		return following;
	}

	public boolean login(String username, String password)
			throws RemoteException {
		if (server.login(username, password, this)) {
			this.username = username;
			followers = server.getFollowers(this);
			following = server.getFollowing(this);
			return true;
		}
		return false;
	}

	public boolean signup(String username, String firstName, String lastName,
			String password, String email) throws RemoteException {
		if (server.signUp(username, firstName, lastName, password, email, this)) {
			this.username = username;
			this.name = firstName + " " + lastName;
			return true;
		}
		return false;
	}

	@Override
	public ArrayList<Tweets> getTweets() throws RemoteException {
		return server.getTweets(this);
	}

	public ArrayList<Tweets> getMyTweets(String username) {
		try {
			return server.getMyTweets(username);
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return null;
	}

	public void toggleFollowUser(String followUserName) {
		try {
			server.toggleFollowUser(this, followUserName);
			this.following = server.getFollowing(this);
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public void logout(){
		try {
			server.logout(this);
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public void tweet(String message){
		try {
			server.tweet(this, message);
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public ArrayList<Users> searchForUsers(String searchText){
		ArrayList<Users> users = null;
		try {
			users = server.searchForUsers(this, searchText);
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		return users;
		
	}
	
	public ArrayList<Tweets> searchForTags(String tag){
		ArrayList<Tweets> tweets = null;
		try {
			tweets = server.searchForTags(tag);
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return tweets;
	}

	public ArrayList<Tweets> getPrivateMessages(){
		ArrayList<Tweets> privateMessages = null;
		
		try {
			privateMessages = server.getPrivateMessages(this);
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		return null;
		
	}

	public void reTweet(Integer messageID){
		try {
			server.reTweet(this, messageID);
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public void sendPrivateMessage(String  destUsername, String message){
		try {
			server.sendPrivateMessage(this, destUsername, message);
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
}
