package rmi;
import java.rmi.Remote;
import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.HashMap;

/**
 * This interface extends the class Remote and specifies a remote accessible
 * method geTweet, which can be accessed externally and enable passing messages
 * on the fly.
 */
public interface ClientInterface extends Remote {

	/**
	 * 
	 * @return: The username of the current client.
	 * @throws RemoteException
	 */
	public String getUsername() throws RemoteException;

	/**
	 * This method is used by the launcher to set the username of the current client
	 * 
	 * @param username: The username of the user which will be set
	 * @throws RemoteException
	 */
	public void setUsername(String username) throws RemoteException;
	
	/**
	 * get the name of the user
	 * 
	 * @return: the name of the user
	 * @throws RemoteException
	 */
	public String getName() throws RemoteException;

	/**
	 * This method is used by the server such that when the user logs in with
	 * his username, his name is set by the server to access the user name fast
	 * 
	 * @param name: The name of the user which will be set
	 * @throws RemoteException
	 */
	public void setName(String name) throws RemoteException;

	/**
	 * This method is used to get a tweet published by a followed user, and its
	 * used by the server to send the tweet directly after a followed user
	 * tweets.
	 * 
	 * @param username
	 *            : get the username of the user that tweeted the message to
	 *            display it
	 * @param name
	 *            : get the name of the user that tweeted the message to display
	 *            it
	 * @param message
	 *            : get the content of the tweet to display it
	 * @param date
	 *            : get the date of the tweet to display it
	 * @throws RemoteException
	 */
	public void getTweet(Users u, String message,
			String date) throws RemoteException;
	
	public  ArrayList<Tweets> getTweets() throws RemoteException;
	/**
	 * This method is used to get a retweet published by a followed user, and
	 * its used by the server to send the retweet directly after a followed user
	 * tweets.
	 * 
	 * @param username
	 *            : The username of the user that retweeted the message
	 * @param name
	 *            : The name of the user that retweeted the message
	 * @param originalUser
	 *            : The username of the user who first tweeted the message
	 * @param originalName
	 *            : The name of the user who first tweeted the message
	 * @param message
	 *            : The content of the tweet
	 * @param date
	 *            : The date of the message where it was retweeted
	 * @throws RemoteException
	 */
	public void getReTweet(Users u, String originalUser,
			String originalName, String message, String date)
			throws RemoteException;

	/**
	 * This method is used to get a private message sent from a user, and its
	 * used by the server to send a private message directly to the intended
	 * user.
	 * 
	 * @param username
	 *            : get the username of the user that sent the private message.
	 * @param name
	 *            : get the name of the user that tweeted the message to display
	 *            it
	 * @param message
	 *            : get the content of the message to display it
	 * @param date
	 *            : get the date of the message to display it
	 * @throws RemoteException
	 */
	public void getPrivateMessage(Users u, String message,
			String date) throws RemoteException;


	public void followUser(Users u) throws RemoteException;

	/**
	 * This method is used by the server such that when this client tweets a
	 * message, the server will get the followers users from the client to send
	 * them the tweet.
	 */
	public ArrayList<Users> getFollowers() throws RemoteException;
	
	/**
	 * This method is used to get the followed users by the client.
	 */
	public ArrayList<Users> getFollowing() throws RemoteException;
	
	
}
