package rmi;

import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.rmi.server.UnicastRemoteObject;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;

import java.sql.*;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

/**
 * This class implements the interface ServerInterface and extends the class
 * UnicastRemoteObject. Its constructor and methods can be accessed remotely.
 */
public class Server extends UnicastRemoteObject implements ServerInteface {

	private static final long serialVersionUID = 1L;
	// The connection to the MySQL database
	Connection conx;
	public static final String DBUSER = "root";
	public static final String DBPASSWORD = "p@ssw0rd";
	public static final String DBLOCATION = "localhost:3306/twitter";

	/** Maintaining a list of available clients */
	ArrayList<ClientInterface> clients;

	public Server() throws RemoteException {
		clients = new ArrayList<ClientInterface>();
	}

	/**
	 * Method used to connect to the mysql database and returns the connection
	 */
	@Override
	public void connectToServer() throws RemoteException {

		try {
			// This will load the MySQL driver, each DB has its own driver
			Class.forName("com.mysql.jdbc.Driver");
			// Setup the connection with the DB
			conx = DriverManager.getConnection("jdbc:mysql://" + DBLOCATION
					+ "?" + "user=" + DBUSER + "&password=" + DBPASSWORD);

		} catch (Exception e) {
			System.out.println("Error: " + e);
		}
	}

	/**
	 * Checks if the username and password are in the database, returns true if
	 * values are valid and false otherwise.
	 */
	@Override
	public boolean login(String username, String password,
			ClientInterface client) throws RemoteException {

		// Statements allow to issue SQL queries to the database
		try {
			Statement statement = conx.createStatement();
			// Result set get the result of the SQL query
			ResultSet resultSet = statement
					.executeQuery("select ID,concat(FirstName,' ',LastName) as Name  from Users where UserName='"
							+ username + "' and Password='" + password + "'");

			// Checks if the sql query has no results then the entrys are wrong
			// and we return false, otherwise we add the client to the clients
			// list and return true
			if (!resultSet.isBeforeFirst()) {
				return false;
			}
			clients.add(client);
			resultSet.next();
			client.setName(resultSet.getString("Name"));
			return true;

		} catch (Exception e) {
			e.printStackTrace();
			return false;
		}

	}

	/**
	 * Insert into the database a new user, it returns false if the username is
	 * already taken, returns true otherwise
	 */
	@Override
	public boolean signUp(String username, String firstName, String lastName,
			String password, String email, ClientInterface client)
			throws RemoteException {

		// Statements allow to issue SQL queries to the database
		try {
			Statement statement = conx.createStatement();

			// Check if the username is already taken
			ResultSet resultSet = statement
					.executeQuery("select * from Users where UserName='"
							+ username + "'");

			if (resultSet.isBeforeFirst()) {
				return false;
			}

			// Execute insert statement inside the database to add a new user,
			// and also add the user to the clients list
			statement
					.execute("insert into Users(UserName,FirstName,LastName,Password,Email)values ('"
							+ username
							+ "','"
							+ firstName
							+ "','"
							+ lastName
							+ "','" + password + "','" + email + "')");
			clients.add(client);
			return true;

		} catch (Exception e) {
			e.printStackTrace();
			return false;
		}

	}

	/**
	 * Insert a new tweet from a username in the databse and send the message to
	 * active Users
	 */
	@Override
	public void tweet(ClientInterface client, String message)
			throws RemoteException {
		// Statements allow to issue SQL queries to the database
		try {
			Statement statement = conx.createStatement();

			// Get the ID of the User to insert it into the tweet table
			ResultSet rs = statement
					.executeQuery("select ID from Users where UserName='"
							+ client.getUsername() + "'");
			rs.next();
			int userID = rs.getInt("ID");
			// Execute insert statement inside the database to add a new tweet
			statement.execute("insert into Tweets(Message,Date,UserID)"
					+ "values ('" + message + "',NOW()," + userID + ")");

		} catch (Exception e) {
			e.printStackTrace();
		}

		/**
		 * This piece of code is used to deliver the tweet directly to actibve
		 * clients. First, we get all the Users that follows the user who
		 * tweeted and then see if the follower is in the list of clients (to
		 * check if he is active or not), then if he is active we deliver the
		 * tweet to him through his interface method getTweet
		 */


		for (Users u: client.getFollowers()) {
			for (ClientInterface c : clients) {
				if (c.getUsername().equals(u.getUsername())) {
					String timeStamp = new SimpleDateFormat(
							"yyyy/MM/dd HH:mm:ss").format(Calendar
							.getInstance().getTime());
					c.getTweet(u, message,
							timeStamp);
				}
			}
		}

	}

	/**
	 * Insert a re-tweet from a username in the database and send the message to
	 * active Users
	 */
	@Override
	public void reTweet(ClientInterface client, Integer messageID)
			throws RemoteException {
		String originalUser = "", originalName = "", message = "";
		// Statements allow to issue SQL queries to the database
		try {
			Statement statement = conx.createStatement();

			// Get the ID of the User to insert it into the forwarded message
			// table
			ResultSet rs = statement
					.executeQuery("select ID from Users where UserName='"
							+ client.getUsername() + "'");
			rs.next();
			int userID = rs.getInt("ID");
			// Execute insert statement inside the database to add a retweet
			statement
					.execute("insert into Retweets(MessageID,Date,UserID)"
							+ "values ('" + messageID + "',NOW()," + userID

							+ ")");
			// Result set used to get the original user of the tweet and the
			// message content from the messageID
			ResultSet rs2 = statement
					.executeQuery("select UserName,concat(FirstName,' ',LastName) as Name,Message from Users inner join Tweets on Tweets.UserID = Users.ID where Tweets.ID ="
							+ messageID);
			rs2.next();
			originalUser = rs2.getString("UserName");
			originalName = rs2.getString("Name");
			message = rs2.getString("Message");

		} catch (Exception e) {
			e.printStackTrace();
		}

		for (Users u:client.getFollowing()) {
			for (ClientInterface c : clients) {
				if (c.getUsername().equals(u.getUsername())) {
					String timeStamp = new SimpleDateFormat(
							"yyyy/MM/dd HH:mm:ss").format(Calendar
							.getInstance().getTime());

					c.getReTweet(u, originalUser, originalName, message,
							timeStamp);
				}
			}
		}

	}

	/** Send a new private tweet from a user to the destination user. */
	@Override
	public void sendPrivateMessage(ClientInterface client, String destUsername,
			String message) throws RemoteException {
		// Statements allow to issue SQL queries to the database
		try {
			Statement statement = conx.createStatement();

			// Get the ID of the User to insert it into the tweet table
			ResultSet rs = statement
					.executeQuery("select ID from Users where UserName='"
							+ client.getUsername() + "'");
			rs.next();
			int userID = rs.getInt("ID");
			// Get the ID of the destination user
			rs = statement
					.executeQuery("select ID from Users where UserName='"
							+ destUsername + "'");
			rs.next();
			int destUserID = rs.getInt("ID");
			// Execute insert statement inside the database to add a new private
			// message
			statement
					.execute("insert into Tweets(Message,Date,UserID,ToUserID)"
							+ "values ('"
							+ message
							+ "',NOW(),"
							+ userID
							+ ","
							+ destUserID + ")");

		} catch (Exception e) {
			e.printStackTrace();
		}
		/**
		 * This code is used to search if the destination user is online(inside
		 * the clients list) to send the message directly to him.
		 */
		for (ClientInterface c : clients) {
			if (c.getUsername().equals(destUsername)) {
				String timeStamp = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss")
						.format(Calendar.getInstance().getTime());
				c.getPrivateMessage(new Users(c.getUsername(), c.getName()),
						message, timeStamp);
			}
		}
	}

	/**
	 * This method is used to return a list of messages containing a tag
	 * searched by a user.
	 */
	@Override
	public ArrayList<Tweets> searchForTags(String tag) throws RemoteException {

		// Statements allow to issue SQL queries to the database
		try {
			Statement statement = conx.createStatement();
			ArrayList<Tweets> tweets = new ArrayList<Tweets>();
			// query used to search for tags found in the tweets in the middle
			// of the text or at the end of the text
			ResultSet rst = statement
					.executeQuery("select Tweets.ID as MessageID, UserName,concat(FirstName,' ',LastName) as OriginalTweeter,Message,Date" +
							" from Tweets inner join Users on Users.ID=Tweets.UserID where Message like '%#"
							+ tag
							+ " %' or Message like '%#"
							+ tag
							+ "%' and ToUserID is null order by Date desc");
			while (rst.next())
				tweets.add(new Tweets(rst.getString("MessageID"), rst
						.getString("UserName"), rst
						.getString("OriginalTweeter"), rst.getString("Message"), rst
						.getString("Date"), null));
			return tweets;
		} catch (SQLException e) {
			e.printStackTrace();
		}

		return null;
	}

	/**
	 * When the user first login, the client will call this method to get all
	 * previous tweets that happened from his followings. This method even
	 * returns the Retweets of the following Users
	 */
	@Override
	public ArrayList<Tweets> getTweets(ClientInterface client)
			throws RemoteException {

		// Statements allow to issue SQL queries to the database
		try {
			Statement statement = conx.createStatement();
			ArrayList<Tweets> tweets = new ArrayList<Tweets>();
			ResultSet rs = statement
					.executeQuery("select ID from Users where UserName = '"
							+ client.getUsername() + "'");

			rs.next();
			// get ID of the username
			int UserId = rs.getInt("ID");

			/**
			 * to display all non private tweets and Retweets of the following
			 * Users by the current user ordered by date in descending order
			 */
			String query = "select  Tweets.ID as MessageID, UserName,concat(FirstName,' ',LastName) as OriginalTweeter,Message,Date ,null as RetweetedBy"
					+ "	from Tweets inner join Users on Tweets.UserID=Users.ID"
					+ " where UserID in (select UserID from Followers where FollowerUserID ="
					+ UserId
					+ " ) and ToUserID IS NULL"
					+ " UNION"
					+ " select Retweets.MessageID as MessageID,(select UserName from Users as t1 where wts.UserID=t1.ID)," +
					"(select concat(t1.FirstName,' ',t1.LastName) from Users as t1 where wts.UserID=t1.ID),"
					+ " Message,Retweets.Date ,concat(t2.FirstName,' ',t2.LastName) "
					+ " from Retweets inner join Users as t2 on Retweets.UserID=t2.ID"
					+ " inner join Tweets as wts on wts.ID=Retweets.MessageID"
					+ " where Retweets.UserId in (select UserID from Followers where FollowerUserID = "
					+ UserId + ")" + " order by Date desc";

			ResultSet rst = statement.executeQuery(query);

			while (rst.next())
				tweets.add(new Tweets(rst.getString("MessageID"), rst
						.getString("UserName"), rst
						.getString("OriginalTweeter"),
						rst.getString("Message"), rst.getString("Date"), rst
								.getString("RetweetedBy")));
			return tweets;
		} catch (SQLException e) {
			e.printStackTrace();
		}

		return null;
	}

	@Override
	public ArrayList<Tweets> getMyTweets(String username)
			throws RemoteException {
		try {
			Statement statement = conx.createStatement();
			ArrayList<Tweets> tweets = new ArrayList<Tweets>();
			ResultSet rs = statement
					.executeQuery("select ID from Users where UserName = '"
							+ username + "'");

			rs.next();
			// get ID of the username
			int UserId = rs.getInt("ID");

			/**
			 * to display all non private tweets and Retweets of the following
			 * Users by the current user ordered by date in descending order
			 */
			String query = "select  Tweets.ID as MessageID, UserName,concat(FirstName,' ',LastName) as OriginalTweeter,Message,Date"
					+ "	from Tweets inner join Users on Tweets.UserID=Users.ID"
					+ " where UserID ="
					+ UserId
					+ "  and ToUserID IS NULL order by Date desc";

			ResultSet rst = statement.executeQuery(query);

			while (rst.next())
				tweets.add(new Tweets(rst.getString("MessageID"), rst
						.getString("UserName"), rst
						.getString("OriginalTweeter"),
						rst.getString("Message"), rst.getString("Date"), null));
			return tweets;
		} catch (SQLException e) {
			e.printStackTrace();
		}

		return null;
	}

	/** Get private messages for a given user */
	@Override
	public ArrayList<Tweets> getPrivateMessages(ClientInterface client)
			throws RemoteException {
		// Statements allow to issue SQL queries to the database
		try {
			Statement statement = conx.createStatement();
			ArrayList<Tweets> tweets = new ArrayList<Tweets>();
			ResultSet rs = statement
					.executeQuery("select ID from Users where UserName = '"
							+ client.getUsername() + "'");

			rs.next();
			// get ID of the username
			int UserId = rs.getInt("ID");

			/**
			 * to display all non private tweets and Retweets of the following
			 * Users by the current user ordered by date in descending order
			 */
			String query = "select Tweets.ID as MessageID, UserName,concat(FirstName,' ',LastName) as OriginalTweeter, Message,Date"
					+ " from Tweets inner join Users on Tweets.UserID=Users.ID"
					+ " where Tweets.ToUserID="
					+ UserId
					+ ""
					+ " order by Date desc";

			ResultSet rst = statement.executeQuery(query);

			while (rst.next())
				tweets.add(new Tweets(rst.getString("MessageID"), rst
						.getString("UserName"), rst
						.getString("OriginalTweeter"),
						rst.getString("Message"), rst.getString("Date"), null));
			return tweets;
		} catch (SQLException e) {
			e.printStackTrace();
		}

		return null;
	}

	/**
	 * This method is a local method which is used for removing a connection
	 * from the ArrayList of all connections.
	 */
	@Override
	public void logout(ClientInterface client) throws RemoteException {
		clients.remove(client);
	}

	/**
	 * Get followers from the database
	 */
	@Override
	public ArrayList<Users> getFollowers(ClientInterface client)
			throws RemoteException {
		// Statements allow to issue SQL queries to the database
		ArrayList<Users> followers = new ArrayList<Users>();
		try {

			Statement statement = conx.createStatement();

			ResultSet rs = statement
					.executeQuery("select ID from Users where UserName = '"
							+ client.getUsername() + "'");

			rs.next();
			// get ID of the username
			int UserId = rs.getInt("ID");
			// to display all non private tweets of the followed Users by the
			// current user
			ResultSet rst = statement
					.executeQuery("select ID,UserName,concat(FirstName,' ',LastName) as Name,Status from Users where "
							+ "ID in (select FollowerUserID from Followers where UserID ="
							+ UserId + " )");
			

			while (rst.next()) {
				Statement statement2 = conx.createStatement();
				ResultSet rsFollow = statement2
						.executeQuery("select * from Followers where FollowerUserID = "
								+ UserId + " and UserID=" + rst.getString("ID"));

				boolean followed = true;
				if (!rsFollow.isBeforeFirst()) {
					followed = false;
				}
				
				followers.add(new Users(rst.getString("UserName"), rst.getString("Name"),rst.getString("Status"), followed));
			}
		} catch (SQLException e) {
			e.printStackTrace();
		} catch (RemoteException e) {
			e.printStackTrace();
		}
		return followers;
	}

	/**
	 * Get following from the database
	 */
	@Override
	public ArrayList<Users> getFollowing(ClientInterface client)
			throws RemoteException {
		// Statements allow to issue SQL queries to the database
		ArrayList<Users> following = new ArrayList<Users>();
		try {

			Statement statement = conx.createStatement();

			ResultSet rs = statement
					.executeQuery("select ID from Users where UserName = '"
							+ client.getUsername() + "'");

			rs.next();
			// get ID of the username
			int UserId = rs.getInt("ID");
			// to display all non private tweets of the followed Users by the
			// current user
			ResultSet rst = statement
					.executeQuery("select UserName,concat(FirstName,' ',LastName) as Name,Status from Users where "
							+ "ID in (select UserID from Followers where FollowerUserID ="
							+ UserId + " )");

			while (rst.next()) {
				following.add(new Users(rst.getString("UserName"), rst.getString("Name"),rst.getString("Status"), true));
			}
		} catch (SQLException e) {
			e.printStackTrace();
		} catch (RemoteException e) {
			e.printStackTrace();
		}
		return following;
	}

	/**
	 * This method is used for a user to be able to search for other Users using
	 * their username or their name
	 */
	@Override
	public ArrayList<Users> searchForUsers(ClientInterface client,
			String searchText) throws RemoteException {
		// Statements allow to issue SQL queries to the database
		try {
			Statement statement = conx.createStatement();
			Statement statement1 = conx.createStatement();
			ArrayList<Users> Users = new ArrayList<Users>();
			ResultSet rs = null;
			String[] split = searchText.split(" ");

			if (split.length == 1)
				rs = statement
						.executeQuery("select Users.ID as ID,UserName,concat(FirstName,' ',LastName) as Name,Status from Users where UserName like '%"
								+ searchText
								+ "%' or FirstName like '%"
								+ searchText
								+ "%' or LastName like '%"
								+ searchText + "%'");
			else
				rs = statement
						.executeQuery("select Users.ID as ID,UserName,concat(FirstName,' ',LastName) as Name,Status from Users where FirstName like '%"
								+ split[0]
								+ "%' and LastName like '%"
								+ split[1] + "%'");
			while (rs.next()) {

				ResultSet rsUser = statement1
						.executeQuery("select ID from Users where UserName = '"
								+ client.getUsername() + "'");
				rsUser.next();
				// get ID of the username
				int UserId = rsUser.getInt("ID");

				ResultSet rsFollow = statement1
						.executeQuery("select * from Followers where FollowerUserID = "
								+ UserId + " and UserID=" + rs.getString("ID"));

				boolean followed = true;
				if (!rsFollow.isBeforeFirst()) {
					followed = false;
				}

				Users.add(new Users(rs.getString("UserName"), rs
						.getString("Name"), rs.getString("Status"), followed));
			}
			return Users;
		} catch (SQLException e) {
			e.printStackTrace();
		}

		return null;
	}

	@Override
	public void toggleFollowUser(ClientInterface client, String followUserName)
			throws RemoteException {

		// Statements allow to issue SQL queries to the database
		try {
			// get id of followed User
			Statement statement = conx.createStatement();
			// Result set get the result of the SQL query
			ResultSet resultSet = statement
					.executeQuery("select ID from Users where UserName='"
							+ followUserName + "'");

			resultSet.next();
			String followUserID = resultSet.getString("ID");
			// get ID if current user
			resultSet = statement
					.executeQuery("select ID from Users where UserName='"
							+ client.getUsername() + "'");

			resultSet.next();
			String userID = resultSet.getString("ID");

			resultSet = statement
					.executeQuery("select * from Followers where UserID = "
							+ followUserID + " and FollowerUserID=" + userID);
			// Check if we need to add a new follow or delete existing one
			if (!resultSet.isBeforeFirst()) {
				statement
						.execute("insert into Followers (UserID,FollowerUserID) values ("
								+ followUserID + "," + userID + ")");
				return;
			}
			statement.execute("delete from Followers where UserID ="
					+ followUserID + " and FollowerUserID= " + userID);

		} catch (Exception e) {
			e.printStackTrace();
		}

	}

	public static void main(String[] args) {
		/**
		 * Bound the server to rmi.
		 */
		try {
			ServerInteface server = new Server();
			server.connectToServer();
			Registry registry = LocateRegistry.createRegistry(5010);
			registry.rebind("Server", server);
			System.out.println("Server bound");
		} catch (Exception ex) {

		}
	}

}