package rmi;
import java.rmi.Remote;
import java.rmi.RemoteException;
import java.sql.*;
import java.util.ArrayList;
import java.util.HashMap;

/**
 * The Server interface where all the main methods of the application resides
 * 
 * @author dv6
 * 
 */
public interface ServerInteface extends Remote {

	public void connectToServer() throws RemoteException;

	/**
	 * Checks if the username and password are in the database, returns true if
	 * values are valid and false otherwise.
	 * 
	 * @param username
	 *            : The username provided by the user to login
	 * @param password
	 *            : The password provided by the user to login
	 * @param client
	 *            : The client object is passed inorder to put the client in the
	 *            clients list to see who of the users are online
	 * @return: boolean whether the user's credentials are correct.
	 * @throws RemoteException
	 */
	public boolean login(String username, String password,
			ClientInterface client) throws RemoteException;

	public void logout(ClientInterface client) throws RemoteException;
	/**
	 * Insert into the database a new user, it returns false if the username is
	 * already taken, returns true otherwise
	 * 
	 * @param username
	 *            : The username of the user he wants to create which will be
	 *            checked
	 * @param firstName
	 *            :The firstname of the user which will be saved
	 * @param lastName
	 *            : The lastname of the user which will be saved
	 * @param password
	 *            : The password of the user which will be saved
	 * @param email
	 *            : The firstname of the user which will be saved
	 * @param client
	 *            : The client object is passed inorder to put the client in the
	 *            clients list to see who of the users are online.
	 * 
	 * @return: boolean whether the username is already taken or not
	 * @throws RemoteException
	 */
	public boolean signUp(String username, String firstName, String lastName,
			String password, String email, ClientInterface client)
			throws RemoteException;

	/**
	 * Insert a new tweet from a username in the database and send the message
	 * to active users
	 * 
	 * @param client
	 *            : The client interface of the user tweeting inorder to get his
	 *            followers directly from his list
	 * @param message
	 *            : The content of the tweet message
	 * @throws RemoteException
	 */
	public void tweet(ClientInterface client, String message)
			throws RemoteException;

	/**
	 * Insert a re-tweet from a username in the database and send the message to
	 * active users
	 * 
	 * @param client
	 *            : The client interface of the user tweeting inorder to get his
	 *            followers directly from his list
	 * @param messageID
	 *            : Get which message the user is retweeting
	 * @throws RemoteException
	 */
	public void reTweet(ClientInterface client, Integer messageID)
			throws RemoteException;;

	/**
	 * Send a new private tweet from a user to the destination user.
	 * 
	 * @param client
	 *            : The Client who is sending the private message
	 * @param destUsername
	 *            : The user who will receive the message
	 * @param message
	 *            : The content of the message
	 * @throws RemoteException
	 */
	public void sendPrivateMessage(ClientInterface client, String destUsername,
			String message) throws RemoteException;

	/**
	 * Get private messages for a given user
	 * 
	 * @param client:		The client that the private messages are sent to.
	 * @throws RemoteException
	 */
	public ArrayList<Tweets> getPrivateMessages(ClientInterface client)
			throws RemoteException;

	/**
	 * This method is used to return a list of messages containing a tag
	 * searched by a user.
	 * 
	 * @param tag
	 *            : The searched tag by a user
	 * @return: A list of messages containing the tag.
	 * @throws RemoteException
	 */
	public  ArrayList<Tweets> searchForTags(String tag) throws RemoteException;
	/**
	 * When the user first login, the client will call this method to get all
	 * previous tweets that happened from his followings. This method even
	 * returns the retweets of the following users.
	 * 
	 * @param client
	 *            : The client that issued this method inorder to search inside
	 *            his following list to get from them their tweets.
	 * @return: ResultSet containing the tweets and retweets from the following users.
	 * @throws RemoteException
	 */
	public  ArrayList<Tweets> getTweets(ClientInterface client) throws RemoteException;
	/**
	 * Get tweets that is done my the passed username
	 * @param username
	 * @return arraylist of tweets done by the passed user
	 * @throws RemoteException
	 */
	public  ArrayList<Tweets> getMyTweets(String username) throws RemoteException;
/**
 * @param client:	The client of which his followers should be returned
 * @return:			Return the followers of the given user
 * @throws RemoteException
 */
	public ArrayList<Users> getFollowers(ClientInterface client)
			throws RemoteException;
	/**
	 * @param client:	The client of which his followings should be returned
	 * @return:			Return the followings of the given user
	 * @throws RemoteException
	 */
	public ArrayList<Users> getFollowing(ClientInterface client)
			throws RemoteException;

	/**
	 * This method is used for a user to be able to search for other users using their username or their name
	 * @param client:	The client searching for users to see if he is following th elisted users or not
	 * @param searchText:	The text which contains the username or the name of the user searched for.
	 * @return:				Return array list of class Users
	 * @throws RemoteException
	 */
	public  ArrayList<Users> searchForUsers(ClientInterface client,String searchText) throws RemoteException;

	public void toggleFollowUser(ClientInterface client,String followUserName) throws RemoteException;
}
